/*
****************************************************************************
*
* MODULE:       d.legend - file vector.c
* AUTHOR(S):    jachym cepicky jachym.cepicky @ centrum.cz http://les-ejk.cz
* PURPOSE:      For drawing vector legend
* COPYRIGHT:    (C) 2005 by the GRASS Development Team
*
*               This program is free software under the GNU General Public
*   	    	License (>=v2). Read the file COPYING that comes with GRASS
*   	    	for details.
*
*****************************************************************************/



/* system header files */
#include <stdio.h> 
#include <ctype.h> 
                    
#include <string.h> 
#include <stdlib.h> 

/* my header files */
#include "gis.h"
#include "display.h"
#include "raster.h"
#include "display.h"
#include "Vect.h"
#include "dbmi.h"
#include "colors.h"
#include "symbol.h"
#include "glocale.h"

#include "vector.h"

/*
 * saves vector categories and colors to struct vcat_clrs
 * returns number of categories
 */
int vcats_and_colors (struct Vcat_clrs *vcat_clrs, char *map_name, char *mapset, char *layer, char *vlegend_column)
{
    int i = 0;
    char buff[512];
    dbDriver *driver;
    dbString stmt, valstr;
    dbCursor cursor;
    dbTable  *table;
    dbColumn *column;
    struct line_cats *Cats;
    struct field_info *fi;
    int nrows = 0, row, cat, more, nvcats = 0, new_cat = 0, ret;    
    struct Map_info Vmap;
    int vlevel;
    struct cat_list *Clist;
    char colorstring[12], vcat[126]; 
    int red, green, blue;

    vlevel = Vect_open_old (&Vmap, map_name, mapset);

    Clist = Vect_new_cat_list ();
    Clist->field = atoi (layer);

    if ((fi = Vect_get_field(&Vmap, Clist->field)) == NULL)
      G_fatal_error (_("Layer <%d> doesn't exist!"), Clist->field);

    Cats = Vect_new_cats_struct ();
    driver = db_start_driver_open_database ( fi->driver, fi->database );
    if ( driver == NULL )
        G_fatal_error (_("Cannot open database %s by driver %s"), fi->database, fi->driver );

    nrows = Vect_get_num_areas(&Vmap);
    
    /* for each row */
    for ( row = 1; row <= nrows; row++ ) {
    cat=Vect_get_area_cat ( &Vmap, row, Clist->field );
    
        if( cat >= 0 ){

            /* getting color */
            db_init_string (&stmt);
            db_init_string (&valstr);
            sprintf ( buff, "select %s, %s from %s where %s = %d ORDER BY grassrgb" , "GRASSRGB", vlegend_column , fi->table,  fi->key, cat);
            db_append_string ( &stmt, buff);

            if (db_open_select_cursor(driver, &stmt, &cursor, DB_SEQUENTIAL) != DB_OK)
                G_fatal_error ("Cannot select attributes for area # %d.", row);

            table = db_get_cursor_table (&cursor);
            column = db_get_table_column(table, 0); /* first column */
            
            if(db_fetch (&cursor, DB_NEXT, &more) != DB_OK) 
                continue;
            
            db_convert_column_value_to_string (column, &valstr);
            sprintf (colorstring, "%s", db_get_string(&valstr));

            /* getting cat value */
            column = db_get_table_column(table, 1); /* second column */
            if(db_fetch (&cursor, DB_NEXT, &more) != DB_OK) continue;
            db_convert_column_value_to_string (column, &valstr);
            sprintf (vcat, "%s", db_get_string(&valstr));

            /* is this a new category ? */
            for (i = 0; i < nvcats; i++) {
                if ( strcmp(vcat, vcat_clrs[i].vcat) == 0 )
                    new_cat = 1;
            }

            if (new_cat == 0) {
                if (nvcats > MAX_VLEGEND_CATS - 1) {
                    break;
                }
                vcat_clrs[nvcats].vcat = G_store (vcat);
                
                /* save colors to struct vcolors */
                ret =  G_str_to_color(colorstring, &red, &green, &blue);
                vcat_clrs[nvcats].red = (unsigned char)red;
                vcat_clrs[nvcats].blue = (unsigned char)blue;
                vcat_clrs[nvcats].green = (unsigned char)green;
    
                
                nvcats += 1;
            }

            new_cat = 0;


        } /* /if (cat >= 0) */
    
        /*if (row > 5) return 0;*/

    }   /* /foreach row */

    return nvcats;
}


/*
 * draws legend
 */

int draw_vector_legend(int x0, int x1, int y0, int y1, struct Vcat_clrs *vcat_clrs, char *column, char* map_name, int nvcats, int map_type)
{
    int i = 0;
    int txsiz;
    char buff[1024];
    int step;
    int cur_row = 0, cur_col = 0;
    int x_box[5];
    int y_box[5];

    int x_line[2];
    int y_line[2];

    int horiz = 0;
    
    if (abs(x1-x0) > abs(y1-y0))
        horiz = 1;

    /* settin text size, size must not be grater than 25 or lesser than 10 */
    R_standard_color(BLACK);

    if(horiz) {
       if ((txsiz = (int)((y1-y0)/nvcats)) > 25) {
            txsiz=25;
        }
        else if (txsiz < 10) {
            txsiz=10;
        } 
    }
    else {
        if ((txsiz = (int)(abs((x1-x0)/nvcats))) > 25) {
            txsiz=25;
        }
        else if (txsiz < 10) {
            txsiz=10;
        } 
    }

    step = txsiz;
    txsiz *= 0.8;
    R_text_size(txsiz, txsiz);

        
    /* for each vector category */
    for (i = 0; i < nvcats; i++) {

        cur_row = i*step;
        cur_col = x0+6;
        
        /* White box */
        R_standard_color(WHITE) ;
        cur_row += step;
        R_move_abs(x0+2, (y0+cur_row-1)) ;
        R_cont_rel(0, (2-step)) ;
        R_cont_rel((step-2), 0) ;
        R_cont_rel(0, (step-2)) ;
        R_cont_rel((2-step), 0) ;

        /* Black box */
        /*R_standard_color(BLACK) ;
        R_move_abs(x0+3, (y0+cur_row-2)) ;
        R_cont_rel(0, (4-step)) ;
        R_cont_rel((step-4), 0) ;
        R_cont_rel(0, (step-4)) ;
        R_cont_rel((4-step), 0) ;*/

        /* white background for lines and points*/
        x_box[0] = 0                 ;
        y_box[0] = 0                 ;
        x_box[1] = 0                 ;
        y_box[1] = (int)(5-step) ;
        x_box[2] = (int)(step-5) ;
        y_box[2] = 0                 ;
        x_box[3] = 0                 ;
        y_box[3] = (int)(step-5) ;
        x_box[4] = (int)(5-step) ;
        y_box[4] = 0                 ;

        R_move_abs(x0+4, (y0+cur_row-2)) ;
        R_standard_color(WHITE) ;
        R_polygon_rel(x_box, y_box, 5) ;
        
        R_RGB_color(vcat_clrs[i].red, vcat_clrs[i].green, vcat_clrs[i].blue);


        /* draw legend area or what? */
        switch(map_type) {
            case GV_FACE:
                    G_debug ( 1, "maptype: face" );
            case GV_POINT:
                    G_debug ( 1, "maptype: point" );
            case GV_AREA:
                    G_debug ( 1, "maptype: area" );
                         /* Set up box arrays */
                        x_box[0] = 0                 ;
                        y_box[0] = 0                 ;
                        x_box[1] = 0                 ;
                        y_box[1] = (int)(5-step) ;
                        x_box[2] = (int)(step-5) ;
                        y_box[2] = 0                 ;
                        x_box[3] = 0                 ;
                        y_box[3] = (int)(step-5) ;
                        x_box[4] = (int)(5-step) ;
                        y_box[4] = 0                 ;


                        R_polygon_rel(x_box, y_box, 5) ;
                        break;
            case GV_BOUNDARY:
                        
                        G_debug ( 1, "maptype: boundary" );
                        x_box[0] = (int)txsiz/2-1;
                        y_box[0] = -txsiz;
                        x_box[1] = (int)-txsiz/2+1;
                        y_box[1] = (int)txsiz/2;
                        x_box[2] = (int)txsiz/2;
                        y_box[2] = (int)txsiz/2;
                        x_box[3] = (int)txsiz/2;
                        y_box[3] = (int)-txsiz/2;
                        x_box[4] = (int)-txsiz/2;
                        y_box[4] = (int)-txsiz/2;

                        R_polyline_rel(x_box, y_box, 5) ;
                        break;
                        
            case GV_LINE:

                        G_debug ( 1, "maptype: line" );
                        x_line[0] = 2;
                        y_line[0] = - 2;
                        x_line[1] = (int)(step-7) ;
                        y_line[1] = (int)(-step+7) ;
                        R_polyline_rel(x_line, y_line, 2);
                        break;

        }
        
        /* draw text */ 
        R_standard_color(BLACK) ;
        R_move_abs((int)x0+6+step, (int)(y0+cur_row-step*0.2));
        
        sprintf(buff , "%s\n", vcat_clrs[i].vcat);
        R_text(buff);

    }
    D_add_to_list(G_recreate_command()) ;
     R_close_driver();
    
 
    return i;
}
